==> mongodb-5.0.31.gh.tar.gz: mongo-r5.0.31/src/mongo/db/auth/user_name.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include <iosfwd>
#include <memory>
#include <string>


#include "mongo/base/clonable_ptr.h"
#include "mongo/base/status_with.h"
#include "mongo/base/string_data.h"
#include "mongo/bson/bsonelement.h"
#include "mongo/bson/bsonobjbuilder.h"
#include "mongo/stdx/variant.h"

namespace mongo {

/**
 * Representation of a name of a principal (authenticatable user) in a MongoDB system.
 *
 * Consists of a "user name" part, and a "database name" part.
 */
class UserName {
public:
    UserName() : _splitPoint(0) {}
    UserName(StringData user, StringData dbname);

    /**
     * Parses a string of the form "db.username" into a UserName object.
     * Don't change the logic of this function as it will break stable API version 1.
     */
    static StatusWith<UserName> parse(StringData userNameStr);

    /**
     * These methods support parsing usernames from IDL
     * Don't change the logic of this function as it will break stable API version 1.
     */
    static UserName parseFromVariant(
        const stdx::variant<std::string, mongo::BSONObj>& helloUserName);
    static UserName parseFromBSONObj(const BSONObj& obj);
    static UserName parseFromBSON(const BSONElement& elem);
    void serializeToBSON(StringData fieldName, BSONObjBuilder* bob) const;
    void serializeToBSON(BSONArrayBuilder* bob) const;

    /**
     * Don't change the logic of this function as it will break stable API version 1.
     */
    void appendToBSON(BSONObjBuilder* bob) const;

    /**
     * Don't change the logic of this function as it will break stable API version 1.
     */
    BSONObj toBSON() const;

    /**
     * Serialized length (in bytes) of object returned by toBSON().
     */
    std::size_t getBSONObjSize() const {
        return 4UL +                          // BSONObj size
            1UL + ("user"_sd).size() + 1UL +  // FieldName elem type, FieldName, NULL.
            4UL + getUser().size() + 1UL +    // Length of name data, name data, NULL.
            1UL + ("db"_sd).size() + 1UL +    // DB field elem type, "db", NULL.
            4UL + getDB().size() + 1UL +      // DB value length, DB value, NULL.
            1UL;                              // EOD marker.
    }

    /**
     * Gets the user part of a UserName.
     */
    StringData getUser() const {
        return StringData(_fullName.data(), _splitPoint);
    }

    /**
     * Gets the database name part of a UserName.
     */
    StringData getDB() const {
        if (_fullName.empty()) {
            return _fullName;
        }

        const auto offset = _splitPoint + 1;
        invariant(offset <= _fullName.size());
        return StringData(_fullName.data() + offset, _fullName.size() - offset);
    }

    /**
     * Gets the full unique name of a user as a string, formatted as "user@db".
     */
    const std::string& getFullName() const {
        return _fullName;
    }

    /**
     * Gets the full unambiguous unique name of a user as a string, formatted as "db.user"
     */
    std::string getUnambiguousName() const {
        return str::stream() << getDB() << "." << getUser();
    }

    /**
     * Stringifies the object, for logging/debugging.
     */
    std::string toString() const {
        return getFullName();
    }

    bool operator==(const UserName& rhs) const {
        return _splitPoint == rhs._splitPoint && getFullName() == rhs.getFullName();
    }

    bool operator!=(const UserName& rhs) const {
        return _splitPoint != rhs._splitPoint || getFullName() != rhs.getFullName();
    }

    bool operator<(const UserName& rhs) const {
        return getUser() < rhs.getUser() || (getUser() == rhs.getUser() && getDB() < rhs.getDB());
    }

    template <typename H>
    friend H AbslHashValue(H h, const UserName& userName) {
        return H::combine(std::move(h), userName.getFullName());
    }

private:
    std::string _fullName;  // The full name, stored as a string.  "user@db".
    size_t _splitPoint;     // The index of the "@" separating the user and db name parts.
};

std::ostream& operator<<(std::ostream& os, const UserName& name);

/**
 * Iterator over an unspecified container of UserName objects.
 */
class UserNameIterator {
public:
    class Impl {
    public:
        Impl() = default;
        virtual ~Impl(){};
        std::unique_ptr<Impl> clone() const {
            return std::unique_ptr<Impl>(doClone());
        }
        virtual bool more() const = 0;
        virtual const UserName& get() const = 0;

        virtual const UserName& next() = 0;

    private:
        virtual Impl* doClone() const = 0;
    };

    UserNameIterator() = default;
    explicit UserNameIterator(std::unique_ptr<Impl> impl) : _impl(std::move(impl)) {}

    bool more() const {
        return _impl.get() && _impl->more();
    }
    const UserName& get() const {
        return _impl->get();
    }

    const UserName& next() {
        return _impl->next();
    }

    const UserName& operator*() const {
        return get();
    }
    const UserName* operator->() const {
        return &get();
    }

private:
    clonable_ptr<Impl> _impl;
};


template <typename ContainerIterator>
class UserNameContainerIteratorImpl : public UserNameIterator::Impl {
public:
    UserNameContainerIteratorImpl(const ContainerIterator& begin, const ContainerIterator& end)
        : _curr(begin), _end(end) {}
    ~UserNameContainerIteratorImpl() override {}
    bool more() const override {
        return _curr != _end;
    }
    const UserName& next() override {
        return *(_curr++);
    }
    const UserName& get() const override {
        return *_curr;
    }
    UserNameIterator::Impl* doClone() const override {
        return new UserNameContainerIteratorImpl(*this);
    }

private:
    ContainerIterator _curr;
    ContainerIterator _end;
};

template <typename ContainerIterator>
UserNameIterator makeUserNameIterator(const ContainerIterator& begin,
                                      const ContainerIterator& end) {
    return UserNameIterator(
        std::make_unique<UserNameContainerIteratorImpl<ContainerIterator>>(begin, end));
}

template <typename Container>
UserNameIterator makeUserNameIteratorForContainer(const Container& container) {
    return makeUserNameIterator(container.begin(), container.end());
}

template <typename Container>
Container userNameIteratorToContainer(UserNameIterator it) {
    Container container;
    while (it.more()) {
        container.emplace_back(it.next());
    }
    return container;
}

}  // namespace mongo

==> mongodb-6.0.27.gh.tar.gz: mongo-r6.0.27/src/mongo/db/auth/user_name.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/auth/auth_name.h"

namespace mongo {

class UserName : public AuthName<UserName> {
public:
    static constexpr auto kName = "UserName"_sd;
    static constexpr auto kFieldName = "user"_sd;

    using AuthName::AuthName;

    const std::string& getUser() const {
        return getName();
    }
};

using UserNameIterator = AuthNameIterator<UserName>;
template <typename Container>
using UserNameContainerIteratorImpl = AuthNameContainerIteratorImpl<Container, UserName>;

template <typename ContainerIterator>
UserNameIterator makeUserNameIterator(const ContainerIterator& begin,
                                      const ContainerIterator& end) {
    return UserNameIterator(
        std::make_unique<UserNameContainerIteratorImpl<ContainerIterator>>(begin, end));
}

template <typename Container>
UserNameIterator makeUserNameIteratorForContainer(const Container& container) {
    return makeUserNameIterator(container.begin(), container.end());
}

template <typename Container>
Container userNameIteratorToContainer(UserNameIterator it) {
    Container container;
    while (it.more()) {
        container.emplace_back(it.next());
    }
    return container;
}

}  // namespace mongo
